"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _cliTableau = _interopRequireDefault(require("cli-tableau"));

var _child_process = require("child_process");

var _promptly = _interopRequireDefault(require("promptly"));

var _jsApi = _interopRequireDefault(require("@pm2/js-api"));

var _constants = _interopRequireDefault(require("../../constants"));

var _package = _interopRequireDefault(require("../../../package.json"));

var _CliAuth = _interopRequireDefault(require("./auth-strategies/CliAuth"));

var _WebAuth = _interopRequireDefault(require("./auth-strategies/WebAuth"));

var OAUTH_CLIENT_ID_WEB = "138558311";
var OAUTH_CLIENT_ID_CLI = "0943857435";

var PM2ioHandler = /*#__PURE__*/function () {
  function PM2ioHandler() {
    (0, _classCallCheck2["default"])(this, PM2ioHandler);
  }

  (0, _createClass2["default"])(PM2ioHandler, null, [{
    key: "usePM2Client",
    value: function usePM2Client(instance) {
      this.pm2 = instance;
    }
  }, {
    key: "strategy",
    value: function strategy() {
      switch (process.platform) {
        case "darwin":
          {
            return new _WebAuth["default"]({
              client_id: OAUTH_CLIENT_ID_WEB
            });
          }

        case "win32":
          {
            return new _WebAuth["default"]({
              client_id: OAUTH_CLIENT_ID_WEB
            });
          }

        case "linux":
          {
            var isDesktop = process.env.XDG_CURRENT_DESKTOP || process.env.XDG_SESSION_DESKTOP || process.env.DISPLAY;
            var isSSH = process.env.SSH_TTY || process.env.SSH_CONNECTION;

            if (isDesktop && !isSSH) {
              return new _WebAuth["default"]({
                client_id: OAUTH_CLIENT_ID_WEB
              });
            } else {
              return new _CliAuth["default"]({
                client_id: OAUTH_CLIENT_ID_CLI
              });
            }
          }

        default:
          {
            return new _CliAuth["default"]({
              client_id: OAUTH_CLIENT_ID_CLI
            });
          }
      }
    }
  }, {
    key: "init",
    value: function init() {
      this._strategy = this.strategy();
      /**
       * If you are using a local backend you should give those options :
       * {
       *   services: {
       *    API: 'http://localhost:3000',
       *    OAUTH: 'http://localhost:3100'
       *   }
       *  }
       */

      this.io = new _jsApi["default"]().use(this._strategy);
    }
  }, {
    key: "launch",
    value: function launch(command, opts) {
      var _this = this;

      // first init the strategy and the io client
      this.init();

      switch (command) {
        case "connect":
        case "login":
        case "register":
        case undefined:
        case "authenticate":
          {
            this.authenticate();
            break;
          }

        case "validate":
          {
            this.validateAccount(opts);
            break;
          }

        case "help":
        case "welcome":
          {
            var dt = _fs["default"].readFileSync(_path["default"].join(__dirname, "./pres/welcome"));

            console.log(dt.toString());
            return process.exit(0);
          }

        case "logout":
          {
            this._strategy.isAuthenticated().then(function (isConnected) {
              // try to kill the agent anyway
              _this.pm2.killAgent(function (err) {});

              if (isConnected === false) {
                console.log("".concat(_constants["default"].PM2_IO_MSG, " Already disconnected"));
                return process.exit(0);
              }

              _this._strategy._retrieveTokens(function (err, tokens) {
                if (err) {
                  console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully disconnected"));
                  return process.exit(0);
                }

                _this._strategy.deleteTokens(_this.io).then(function (_) {
                  console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully disconnected"));
                  return process.exit(0);
                })["catch"](function (err) {
                  console.log("".concat(_constants["default"].PM2_IO_MSG_ERR, " Unexpected error: ").concat(err.message));
                  return process.exit(1);
                });
              });
            })["catch"](function (err) {
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to logout: ").concat(err.message));
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
            });

            break;
          }

        case "create":
          {
            this._strategy.isAuthenticated().then(function (res) {
              // if the user isn't authenticated, we make them do the whole flow
              if (res !== true) {
                _this.authenticate();
              } else {
                _this.createBucket(_this.createBucketHandler.bind(_this));
              }
            })["catch"](function (err) {
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to create to the bucket: ").concat(err.message));
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
            });

            break;
          }

        case "web":
          {
            this._strategy.isAuthenticated().then(function (res) {
              // if the user isn't authenticated, we make them do the whole flow
              if (res === false) {
                console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You need to be authenticated to do that, please use: pm2 plus login"));
                return process.exit(1);
              }

              _this._strategy._retrieveTokens(function () {
                return _this.openUI();
              });
            })["catch"](function (err) {
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to open the UI: ").concat(err.message));
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
            });

            break;
          }

        default:
          {
            console.log("".concat(_constants["default"].PM2_IO_MSG_ERR, " Invalid command ").concat(command, ", available : login,register,validate,connect or web"));
            process.exit(1);
          }
      }
    }
  }, {
    key: "openUI",
    value: function openUI() {
      var _this2 = this;

      this.io.bucket.retrieveAll().then(function (res) {
        var buckets = res.data;

        if (buckets.length === 0) {
          return _this2.createBucket(function (err, bucket) {
            if (err) {
              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to connect to the bucket: ").concat(err.message));

              if (bucket) {
                console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can retry using: pm2 plus link ").concat(bucket.secret_id, " ").concat(bucket.public_id));
              }

              console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
              return process.exit(0);
            }

            var targetURL = "https://app.pm2.io/#/bucket/".concat(bucket._id);
            console.log("".concat(_constants["default"].PM2_IO_MSG, " Please follow the popup or go to this URL :"), "\n", "    ", targetURL);

            _this2.open(targetURL);

            return process.exit(0);
          });
        }

        var table = new _cliTableau["default"]({
          style: {
            "padding-left": 1,
            head: ["cyan", "bold"],
            compact: true
          },
          head: ["Bucket name", "Plan type"]
        });
        buckets.forEach(function (bucket) {
          table.push([bucket.name, bucket.credits.offer_type]);
        });
        console.log(table.toString());
        console.log("".concat(_constants["default"].PM2_IO_MSG, " If you don't want to open the UI to a bucket, type 'none'"));
        var choices = buckets.map(function (bucket) {
          return bucket.name;
        });
        choices.push("none");

        _promptly["default"].choose("".concat(_constants["default"].PM2_IO_MSG, " Type the name of the bucket you want to connect to :"), choices, function (err, value) {
          if (value === "none") {
            process.exit(0);
          }

          var bucket = buckets.find(function (bucket) {
            return bucket.name === value;
          });

          if (bucket === undefined) {
            return process.exit(0);
          }

          var targetURL = "https://app.pm2.io/#/bucket/".concat(bucket._id);
          console.log("".concat(_constants["default"].PM2_IO_MSG, " Please follow the popup or go to this URL :"), "\n", "    ", targetURL);

          _this2.open(targetURL);

          return process.exit(0);
        });
      });
    }
  }, {
    key: "validateAccount",
    value: function validateAccount(token) {
      this.io.auth.validEmail(token).then(function (res) {
        console.log("".concat(_constants["default"].PM2_IO_MSG, " Email succesfully validated."));
        console.log("".concat(_constants["default"].PM2_IO_MSG, " You can now proceed and use: pm2 plus connect"));
        return process.exit(0);
      })["catch"](function (err) {
        if (err.status === 401) {
          console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Invalid token"));
          return process.exit(1);
        } else if (err.status === 301) {
          console.log("".concat(_constants["default"].PM2_IO_MSG, " Email succesfully validated."));
          console.log("".concat(_constants["default"].PM2_IO_MSG, " You can now proceed and use: pm2 plus connect"));
          return process.exit(0);
        }

        var msg = err.data ? err.data.error_description || err.data.msg : err.message;
        console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to validate your email: ").concat(msg));
        console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
        return process.exit(1);
      });
    }
  }, {
    key: "createBucketHandler",
    value: function createBucketHandler(err, bucket) {
      if (err) {
        console.trace("".concat(_constants["default"].PM2_IO_MSG_ERR, " Failed to connect to the bucket: ").concat(err.message));

        if (bucket) {
          console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can retry using: pm2 plus link ").concat(bucket.secret_id, " ").concat(bucket.public_id));
        }

        console.error("".concat(_constants["default"].PM2_IO_MSG_ERR, " You can also contact us to get help: contact@pm2.io"));
        return process.exit(0);
      }

      if (bucket === undefined) {
        return process.exit(0);
      }

      console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully connected to bucket ").concat(bucket.name));
      var targetURL = "https://app.pm2.io/#/bucket/".concat(bucket._id);
      console.log("".concat(_constants["default"].PM2_IO_MSG, " You can use the web interface over there: ").concat(targetURL));
      this.open(targetURL);
      return process.exit(0);
    }
  }, {
    key: "createBucket",
    value: function createBucket(cb) {
      var _this3 = this;

      console.log("".concat(_constants["default"].PM2_IO_MSG, " By default we allow you to trial PM2 Plus for 14 days without any credit card."));
      this.io.bucket.create({
        name: "PM2 Plus Monitoring"
      }).then(function (res) {
        var bucket = res.data.bucket;
        console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully created the bucket"));

        _this3.pm2.link({
          public_key: bucket.public_id,
          secret_key: bucket.secret_id,
          pm2_version: _package["default"].version
        }, function (err) {
          if (err) {
            return cb(new Error("Failed to connect your local PM2 to your bucket"), bucket);
          } else {
            return cb(null, bucket);
          }
        });
      })["catch"](function (err) {
        return cb(new Error("Failed to create a bucket: ".concat(err.message)));
      });
    }
    /**
     * Connect the local agent to a specific bucket
     * @param {Function} cb
     */

  }, {
    key: "connectToBucket",
    value: function connectToBucket(cb) {
      var _this4 = this;

      this.io.bucket.retrieveAll().then(function (res) {
        var buckets = res.data;

        if (buckets.length === 0) {
          return _this4.createBucket(cb);
        }

        var table = new _cliTableau["default"]({
          style: {
            "padding-left": 1,
            head: ["cyan", "bold"],
            compact: true
          },
          head: ["Bucket name", "Plan type"]
        });
        buckets.forEach(function (bucket) {
          table.push([bucket.name, bucket.payment.offer_type]);
        });
        console.log(table.toString());
        console.log("".concat(_constants["default"].PM2_IO_MSG, " If you don't want to connect to a bucket, type 'none'"));
        var choices = buckets.map(function (bucket) {
          return bucket.name;
        });
        choices.push("none");

        _promptly["default"].choose("".concat(_constants["default"].PM2_IO_MSG, " Type the name of the bucket you want to connect to :"), choices, function (err, value) {
          if (value === "none") {
            return cb();
          }

          var bucket = buckets.find(function (bucket) {
            return bucket.name === value;
          });

          if (bucket === undefined) {
            return cb();
          }

          _this4.pm2.link({
            public_key: bucket.public_id,
            secret_key: bucket.secret_id,
            pm2_version: _package["default"].version
          }, function (err) {
            return err ? cb(err) : cb(null, bucket);
          });
        });
      });
    }
    /**
     * Authenticate the user with either of the strategy
     * @param {Function} cb
     */

  }, {
    key: "authenticate",
    value: function authenticate() {
      var _this5 = this;

      this._strategy._retrieveTokens(function (err, tokens) {
        if (err) {
          var msg = err.data ? err.data.error_description || err.data.msg : err.message;
          console.log("".concat(_constants["default"].PM2_IO_MSG_ERR, " Unexpected error : ").concat(msg));
          return process.exit(1);
        }

        console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully authenticated"));

        _this5.io.user.retrieve().then(function (res) {
          // const user = res.data;
          _this5.io.user.retrieve().then(function (res) {
            // const tmpUser = res.data;
            console.log("".concat(_constants["default"].PM2_IO_MSG, " Successfully validated"));

            _this5.connectToBucket(_this5.createBucketHandler.bind(_this5));
          });
        });
      });
    }
  }, {
    key: "open",
    value: function open(target, appName, callback) {
      var opener;

      var escape = function escape(s) {
        return s.replace(/"/g, "\\\"");
      };

      if (typeof appName === "function") {
        callback = appName;
        appName = null;
      }

      switch (process.platform) {
        case "darwin":
          {
            opener = appName ? "open -a \"".concat(escape(appName), "\"") : "open";
            break;
          }

        case "win32":
          {
            opener = appName ? "start \"\" ".concat(escape(appName), "\"") : "start \"\"";
            break;
          }

        default:
          {
            opener = appName ? escape(appName) : "xdg-open";
            break;
          }
      }

      if (process.env.SUDO_USER) {
        opener = "sudo -u " + process.env.SUDO_USER + " " + opener;
      }

      return (0, _child_process.exec)("".concat(opener, " \"").concat(escape(target), "\""), callback);
    }
  }]);
  return PM2ioHandler;
}();

exports["default"] = PM2ioHandler;
(0, _defineProperty2["default"])(PM2ioHandler, "pm2", void 0);
(0, _defineProperty2["default"])(PM2ioHandler, "_strategy", void 0);
(0, _defineProperty2["default"])(PM2ioHandler, "io", void 0);