"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _os = _interopRequireDefault(require("os"));

var _debug = _interopRequireDefault(require("debug"));

var _child_process = require("child_process");

var _pm2tIoAgent = _interopRequireDefault(require("pm2t-io-agent"));

var _pm2AxonRpc = _interopRequireDefault(require("pm2-axon-rpc"));

var _forEach = _interopRequireDefault(require("async/forEach"));

var _pm2Axon = _interopRequireDefault(require("pm2-axon"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _Common = _interopRequireDefault(require("./Common"));

var _package = _interopRequireDefault(require("../package.json"));

var _which = _interopRequireDefault(require("./tools/which"));

var _constants = _interopRequireDefault(require("./constants"));

var _Daemon = _interopRequireDefault(require("./Daemon"));

var _VersionCheck = _interopRequireDefault(require("./VersionCheck"));

/**
 * Copyright 2013 the PM2 project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */
var debug = (0, _debug["default"])("pm2t:client");

function noop() {// do nothing
}

var Client = function Client(_opts) {
  var _this = this;

  (0, _classCallCheck2["default"])(this, Client);
  (0, _defineProperty2["default"])(this, "conf", void 0);
  (0, _defineProperty2["default"])(this, "sub_sock", void 0);
  (0, _defineProperty2["default"])(this, "daemon_mode", void 0);
  (0, _defineProperty2["default"])(this, "pm2_home", void 0);
  (0, _defineProperty2["default"])(this, "public_key", void 0);
  (0, _defineProperty2["default"])(this, "secret_key", void 0);
  (0, _defineProperty2["default"])(this, "machine_name", void 0);
  (0, _defineProperty2["default"])(this, "rpc_socket_file", void 0);
  (0, _defineProperty2["default"])(this, "pub_socket_file", void 0);
  (0, _defineProperty2["default"])(this, "client", void 0);
  (0, _defineProperty2["default"])(this, "interactor_process", void 0);
  (0, _defineProperty2["default"])(this, "client_sock", void 0);
  (0, _defineProperty2["default"])(this, "sub", void 0);
  (0, _defineProperty2["default"])(this, "start", function (cb) {
    _this.pingDaemon(function (daemonAlive) {
      if (daemonAlive === true) {
        return _this.launchRPC(function (err, meta) {
          return cb(null, {
            daemon_mode: _this.conf.daemon_mode,
            new_pm2_instance: false,
            rpc_socket_file: _this.rpc_socket_file,
            pub_socket_file: _this.pub_socket_file,
            pm2_home: _this.pm2_home
          });
        });
      }
      /**
       * No Daemon mode
       */


      if (_this.daemon_mode === false) {
        var daemon = new _Daemon["default"]({
          pub_socket_file: _this.conf.DAEMON_PUB_PORT,
          rpc_socket_file: _this.conf.DAEMON_RPC_PORT,
          pid_file: _this.conf.PM2_PID_FILE_PATH,
          ignore_signals: true
        });
        console.log("Launching in no daemon mode");
        daemon.innerStart(function () {
          _pm2tIoAgent["default"].launchAndInteract(_this.conf, {
            machine_name: _this.machine_name,
            public_key: _this.public_key,
            secret_key: _this.secret_key,
            pm2_version: _package["default"].version
          }, function (err, data, interactor_proc) {
            _this.interactor_process = interactor_proc;
          });

          _this.launchRPC(function (err, meta) {
            return cb(null, {
              daemon_mode: _this.conf.daemon_mode,
              new_pm2_instance: true,
              rpc_socket_file: _this.rpc_socket_file,
              pub_socket_file: _this.pub_socket_file,
              pm2_home: _this.pm2_home
            });
          });
        });
        return false;
      }
      /**
       * Daemon mode
       */


      _this.launchDaemon(function (err, child) {
        if (err) {
          _Common["default"].printError(err);

          return cb ? cb(err) : process.exit(_this.conf.ERROR_EXIT);
        }

        if (!process.env.PM2_DISCRETE_MODE) {
          _Common["default"].printOut(_this.conf.PREFIX_MSG + "PM2 Successfully daemonized");
        }

        _this.launchRPC(function (err, meta) {
          return cb(null, {
            daemon_mode: _this.conf.daemon_mode,
            new_pm2_instance: true,
            rpc_socket_file: _this.rpc_socket_file,
            pub_socket_file: _this.pub_socket_file,
            pm2_home: _this.pm2_home
          });
        });
      });
    });
  });
  (0, _defineProperty2["default"])(this, "initFileStructure", function (opts) {
    if (!_fs["default"].existsSync(opts.DEFAULT_LOG_PATH)) {
      try {
        _mkdirp["default"].sync(opts.DEFAULT_LOG_PATH);
      } catch (e) {
        console.error(e.stack || e);
      }
    }

    if (!_fs["default"].existsSync(opts.DEFAULT_PID_PATH)) {
      try {
        _mkdirp["default"].sync(opts.DEFAULT_PID_PATH);
      } catch (e) {
        console.error(e.stack || e);
      }
    }

    if (!_fs["default"].existsSync(opts.PM2_MODULE_CONF_FILE)) {
      try {
        _fs["default"].writeFileSync(opts.PM2_MODULE_CONF_FILE, "{}");
      } catch (e) {
        console.error(e.stack || e);
      }
    }

    if (!_fs["default"].existsSync(opts.DEFAULT_MODULE_PATH)) {
      try {
        _mkdirp["default"].sync(opts.DEFAULT_MODULE_PATH);
      } catch (e) {
        console.error(e.stack || e);
      }
    }

    if (process.env.PM2_DISCRETE_MODE) {
      try {
        _fs["default"].writeFileSync(_path["default"].join(opts.PM2_HOME, "touch"), Date.now().toString());
      } catch (e) {
        debug(e.stack || e);
      }
    }

    if (!process.env.PM2_PROGRAMMATIC && !_fs["default"].existsSync(_path["default"].join(opts.PM2_HOME, "touch"))) {
      (0, _VersionCheck["default"])({
        state: "install",
        version: _package["default"].version
      });

      var dt = _fs["default"].readFileSync(_path["default"].resolve(__dirname, "..", opts.PM2_BANNER));

      console.log(dt.toString());

      try {
        _fs["default"].writeFileSync(_path["default"].join(opts.PM2_HOME, "touch"), Date.now().toString());
      } catch (e) {
        debug(e.stack || e);
      }
    }
  });
  (0, _defineProperty2["default"])(this, "close", function (cb) {
    (0, _forEach["default"])([_this.disconnectRPC.bind(_this), _this.disconnectBus.bind(_this)], function (fn, next) {
      fn(next);
    }, cb);
  });
  (0, _defineProperty2["default"])(this, "launchDaemon", function (opts, cb) {
    if (typeof opts == "function") {
      cb = opts;
      opts = {
        interactor: true
      };
    }

    var ClientJS = _path["default"].resolve(_path["default"].dirname(module.filename), "Daemon.js");

    var node_args = [];
    var out, err; // if (process.env.TRAVIS) {
    //   // Redirect PM2 internal err and out to STDERR STDOUT when running with Travis
    //   out = 1;
    //   err = 2;
    // }
    // else {

    out = _fs["default"].openSync(_this.conf.PM2_LOG_FILE_PATH, "a"), err = _fs["default"].openSync(_this.conf.PM2_LOG_FILE_PATH, "a"); //}

    if (_this.conf.LOW_MEMORY_ENVIRONMENT) {
      node_args.push("--gc-global"); // Does full GC (smaller memory footprint)

      node_args.push("--max-old-space-size=" + Math.floor(_os["default"].totalmem() / 1024 / 1024));
    } // Node.js tuning for better performance
    //node_args.push('--expose-gc'); // Allows manual GC in the code

    /**
     * Add node [arguments] depending on PM2_NODE_OPTIONS env variable
     */


    if (process.env.PM2_NODE_OPTIONS) {
      node_args = node_args.concat(process.env.PM2_NODE_OPTIONS.split(" "));
    }

    node_args.push(ClientJS);

    if (!process.env.PM2_DISCRETE_MODE) {
      _Common["default"].printOut(_this.conf.PREFIX_MSG + "Spawning PM2 daemon with pm2_home=" + _this.pm2_home);
    }

    var interpreter = "node";

    if ((0, _which["default"])("node") == null) {
      interpreter = process.execPath;
    }

    var spawnEnv = Object.assign({}, {
      "SILENT": _this.conf.DEBUG ? !_this.conf.DEBUG : true,
      "PM2_HOME": _this.pm2_home
    }, process.env);
    var child = (0, _child_process.spawn)(interpreter, node_args, {
      detached: true,
      cwd: _this.conf.cwd || process.cwd(),
      env: spawnEnv,
      stdio: ["ipc", out, err]
    });

    function onError(e) {
      console.error(e.message || e);
      return cb ? cb(e.message || e) : false;
    }

    child.once("error", onError);
    child.unref();
    child.once("message", function (msg) {
      var _this2 = this;

      debug("PM2 daemon launched with return message: ", msg);
      child.removeListener("error", onError);
      child.disconnect();

      if (opts && opts.interactor == false) {
        return cb(null, child);
      }

      if (process.env.PM2_NO_INTERACTION == "true") {
        return cb(null, child);
      }
      /**
       * Here the Keymetrics agent is launched automaticcaly if
       * it has been already configured before (via pm2 link)
       */


      _pm2tIoAgent["default"].launchAndInteract(this.conf, {
        machine_name: this.machine_name,
        public_key: this.public_key,
        secret_key: this.secret_key,
        pm2_version: _package["default"].version
      }, function (err, data, interactor_proc) {
        _this2.interactor_process = interactor_proc;
        return cb(null, child);
      });
    });
  });
  (0, _defineProperty2["default"])(this, "pingDaemon", function (cb) {
    var req = _pm2Axon["default"].socket("req", null);

    var client = new _pm2AxonRpc["default"].Client(req);
    debug("[PING PM2] Trying to connect to server");
    client.sock.once("reconnect attempt", function () {
      client.sock.close();
      debug("Daemon not launched");
      process.nextTick(function () {
        return cb(false);
      });
    });
    client.sock.once("error", function (e) {
      if (e.code === "EACCES") {
        _fs["default"].stat(_this.conf.DAEMON_RPC_PORT, function (e, stats) {
          if (stats.uid === 0) {
            // console.error(this.conf.PREFIX_MSG_ERR + "Permission denied, to give access to current user:");
            console.log(new Error(_this.conf.PREFIX_MSG_ERR + "Permission denied, to give access to current user:").stack);
            console.log("$ sudo chown " + process.env.USER + ":" + process.env.USER + " " + _this.conf.DAEMON_RPC_PORT + " " + _this.conf.DAEMON_PUB_PORT);
          } else {
            // console.error(this.conf.PREFIX_MSG_ERR + "Permission denied, check permissions on " + this.conf.DAEMON_RPC_PORT);
            console.log(new Error(_this.conf.PREFIX_MSG_ERR + "Permission denied, check permissions on " + _this.conf.DAEMON_RPC_PORT).stack);
          }

          process.exit(1);
        });
      } else {
        console.error(e.message || e);
      }
    });
    client.sock.once("connect", function () {
      client.sock.once("close", function () {
        return cb(true);
      });
      client.sock.close();
      debug("Daemon alive");
    });
    req.connect(_this.rpc_socket_file);
  });
  (0, _defineProperty2["default"])(this, "launchRPC", function (cb) {
    debug("Launching RPC client on socket file %s", _this.rpc_socket_file);

    var req = _pm2Axon["default"].socket("req", null);

    _this.client = new _pm2AxonRpc["default"].Client(req);

    var connectHandler = function connectHandler() {
      _this.client.sock.removeListener("error", errorHandler);

      debug("RPC Connected to Daemon");

      if (cb) {
        setTimeout(function () {
          cb(null);
        }, 4);
      }
    };

    var errorHandler = function errorHandler(e) {
      _this.client.sock.removeListener("connect", connectHandler);

      if (cb) {
        return cb(e);
      }
    };

    _this.client.sock.once("connect", connectHandler);

    _this.client.sock.once("error", errorHandler);

    _this.client_sock = req.connect(_this.rpc_socket_file);
  });
  (0, _defineProperty2["default"])(this, "disconnectRPC", function (cb) {
    if (!cb) {
      cb = noop;
    }

    if (!_this.client_sock || !_this.client_sock.close) {
      _this.client = null;
      return process.nextTick(function () {
        cb(new Error("SUB connection to PM2 is not launched"));
      });
    }

    if (_this.client_sock.connected === false || _this.client_sock.closing === true) {
      _this.client = null;
      return process.nextTick(function () {
        cb(new Error("RPC already being closed"));
      });
    }

    try {
      var timer; // eslint-disable-line prefer-const

      _this.client_sock.once("close", function () {
        clearTimeout(timer);
        _this.client = null;
        debug("PM2 RPC cleanly closed");
        return cb(null, {
          msg: "RPC Successfully closed"
        });
      });

      timer = setTimeout(function () {
        if (_this.client_sock.destroy) {
          _this.client_sock.destroy();
        }

        _this.client = null;
        return cb(null, {
          msg: "RPC Successfully closed via timeout"
        });
      }, 200);

      _this.client_sock.close();
    } catch (e) {
      debug("Error while disconnecting RPC PM2", e.stack || e);
      return cb(e);
    }

    return false;
  });
  (0, _defineProperty2["default"])(this, "launchBus", function (cb) {
    _this.sub = _pm2Axon["default"].socket("sub-emitter", null);
    _this.sub_sock = _this.sub.connect(_this.pub_socket_file);

    _this.sub_sock.once("connect", function () {
      return cb(null, _this.sub, _this.sub_sock);
    });
  });
  (0, _defineProperty2["default"])(this, "disconnectBus", function (cb) {
    if (!cb) {
      cb = noop;
    }

    if (!_this.sub_sock || !_this.sub_sock.close) {
      _this.sub = null;
      return process.nextTick(function () {
        cb(null, {
          msg: "bus was not connected"
        });
      });
    }

    if (_this.sub_sock.connected === false || _this.sub_sock.closing === true) {
      _this.sub = null;
      return process.nextTick(function () {
        cb(new Error("SUB connection is already being closed"));
      });
    }

    try {
      var timer; // eslint-disable-line prefer-const

      _this.sub_sock.once("close", function () {
        _this.sub = null;
        clearTimeout(timer);
        debug("PM2 PUB cleanly closed");
        return cb();
      });

      timer = setTimeout(function () {
        if (_this.sub_sock.destroy) {
          _this.sub_sock.destroy();
        }

        return cb();
      }, 200);

      _this.sub_sock.close();
    } catch (e) {
      return cb(e);
    }
  });
  (0, _defineProperty2["default"])(this, "getExposedMethods", function (cb) {
    _this.client.methods(cb);
  });
  (0, _defineProperty2["default"])(this, "executeRemote", function (method, app_conf, fn) {
    // stop watch on stop | env is the process id
    if (method.indexOf("stop") !== -1) {
      _this.stopWatch(method, app_conf);
    } else if (method.indexOf("delete") !== -1) {
      // stop watching when process is deleted
      _this.stopWatch(method, app_conf);
    } else if (method.indexOf("kill") !== -1) {
      // stop everything on kill
      _this.stopWatch("deleteAll", app_conf);
    } else if (method.indexOf("restartProcessId") !== -1 && process.argv.indexOf("--watch") > -1) {
      delete app_conf.env.current_conf.watch;

      _this.toggleWatch(method, app_conf);
    }

    if (!_this.client || !_this.client.call) {
      _this.start(function (error) {
        if (error) {
          if (fn) {
            return fn(error);
          }

          console.error(error);
          return process.exit(0);
        }

        if (_this.client) {
          return _this.client.call(method, app_conf, fn);
        }
      });

      return false;
    }

    debug("Calling daemon method pm2:%s on rpc socket:%s", method, _this.rpc_socket_file);
    return _this.client.call(method, app_conf, fn);
  });
  (0, _defineProperty2["default"])(this, "notifyGod", function (action_name, id, cb) {
    _this.executeRemote("notifyByProcessId", {
      id: id,
      action_name: action_name,
      manually: true
    }, function () {
      debug("God notified");
      return cb ? cb() : false;
    });
  });
  (0, _defineProperty2["default"])(this, "killDaemon", function (fn) {
    var timeout; // eslint-disable-line prefer-const

    var quit = function quit() {
      _this.close(function () {
        return fn ? fn(null, {
          success: true
        }) : false;
      });
    }; // under unix, we listen for signal (that is send by daemon to notify us that its shuting down)


    if (process.platform !== "win32") {
      process.once("SIGQUIT", function () {
        debug("Received SIGQUIT from pm2 daemon");
        clearTimeout(timeout);
        quit();
      });
    } else {
      // if under windows, try to ping the daemon to see if it still here
      setTimeout(function () {
        _this.pingDaemon(function (alive) {
          if (!alive) {
            clearTimeout(timeout);
            return quit();
          }
        });
      }, 250);
    }

    timeout = setTimeout(function () {
      quit();
    }, 3000); // Kill daemon

    _this.executeRemote("killMe", {
      pid: process.pid
    });
  });
  (0, _defineProperty2["default"])(this, "toggleWatch", function (method, env, fn) {
    debug("Calling toggleWatch");

    _this.client.call("toggleWatch", method, env, function () {
      return fn ? fn() : false;
    });
  });
  (0, _defineProperty2["default"])(this, "startWatch", function (method, env, fn) {
    debug("Calling startWatch");

    _this.client.call("startWatch", method, env, function () {
      return fn ? fn() : false;
    });
  });
  (0, _defineProperty2["default"])(this, "stopWatch", function (method, env, fn) {
    debug("Calling stopWatch");

    _this.client.call("stopWatch", method, env, function () {
      return fn ? fn() : false;
    });
  });
  (0, _defineProperty2["default"])(this, "getAllProcess", function (cb) {
    // const found_proc = [];
    _this.executeRemote("getMonitorData", {}, function (err, procs) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      return cb(null, procs);
    });
  });
  (0, _defineProperty2["default"])(this, "getAllProcessId", function (cb) {
    // const found_proc = [];
    _this.executeRemote("getMonitorData", {}, function (err, procs) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      return cb(null, procs.map(function (proc) {
        return proc.pm_id;
      }));
    });
  });
  (0, _defineProperty2["default"])(this, "getAllProcessIdWithoutModules", function (cb) {
    // const found_proc = [];
    _this.executeRemote("getMonitorData", {}, function (err, procs) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      var proc_ids = procs.filter(function (proc) {
        return !proc.pm2_env.pmx_module;
      }).map(function (proc) {
        return proc.pm_id;
      });
      return cb(null, proc_ids);
    });
  });
  (0, _defineProperty2["default"])(this, "getProcessIdByName", function (name, force_all, cb) {
    var found_proc = [];
    var full_details = {};

    if (typeof cb === "undefined") {
      cb = force_all;
      force_all = false;
    }

    if (typeof name == "number") {
      name = name.toString();
    }

    _this.executeRemote("getMonitorData", {}, function (err, list) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      list.forEach(function (proc) {
        if (proc.pm2_env.name == name || proc.pm2_env.pm_exec_path == _path["default"].resolve(name)) {
          found_proc.push(proc.pm_id);
          full_details[proc.pm_id] = proc;
        }
      });
      return cb(null, found_proc, full_details);
    });
  });
  (0, _defineProperty2["default"])(this, "getProcessIdsByNamespace", function (namespace, force_all, cb) {
    var found_proc = [];
    var full_details = {};

    if (typeof cb === "undefined") {
      cb = force_all;
      force_all = false;
    }

    if (typeof namespace == "number") {
      namespace = namespace.toString();
    }

    _this.executeRemote("getMonitorData", {}, function (err, list) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      list.forEach(function (proc) {
        if (proc.pm2_env.namespace == namespace) {
          found_proc.push(proc.pm_id);
          full_details[proc.pm_id] = proc;
        }
      });
      return cb(null, found_proc, full_details);
    });
  });
  (0, _defineProperty2["default"])(this, "getProcessByName", function (name, cb) {
    var found_proc = [];

    _this.executeRemote("getMonitorData", {}, function (err, list) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      list.forEach(function (proc) {
        if (proc.pm2_env.name == name || proc.pm2_env.pm_exec_path == _path["default"].resolve(name)) {
          found_proc.push(proc);
        }
      });
      return cb(null, found_proc);
    });
  });
  (0, _defineProperty2["default"])(this, "getProcessByNameOrId", function (nameOrId, cb) {
    var foundProc = [];

    _this.executeRemote("getMonitorData", {}, function (err, list) {
      if (err) {
        _Common["default"].printError("Error retrieving process list: " + err);

        return cb(err);
      }

      list.forEach(function (proc) {
        if (proc.pm2_env.name === nameOrId || proc.pm2_env.pm_exec_path === _path["default"].resolve(nameOrId) || proc.pid === parseInt(nameOrId) || proc.pm2_env.pm_id === parseInt(nameOrId)) {
          foundProc.push(proc);
        }
      });
      return cb(null, foundProc);
    });
  });

  if (!_opts) {
    _opts = {};
  }

  if (!_opts.conf) {
    this.conf = _constants["default"];
  } else {
    this.conf = _opts.conf;
  }

  this.sub_sock = {};
  this.daemon_mode = typeof _opts.daemon_mode === "undefined" ? true : _opts.daemon_mode;
  this.pm2_home = this.conf.PM2_ROOT_PATH;
  this.secret_key = _opts.secret_key;
  this.public_key = _opts.public_key;
  this.machine_name = _opts.machine_name; // Create all folders and files needed
  // Client depends to that to interact with PM2 properly

  this.initFileStructure(this.conf);
  debug("Using RPC file %s", this.conf.DAEMON_RPC_PORT);
  debug("Using PUB file %s", this.conf.DAEMON_PUB_PORT);
  this.rpc_socket_file = this.conf.DAEMON_RPC_PORT;
  this.pub_socket_file = this.conf.DAEMON_PUB_PORT;
} // @breaking change (noDaemonMode has been drop)
// @todo ret err
;

var _default = Client;
exports["default"] = _default;