"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _os = _interopRequireDefault(require("os"));

var _chalk = _interopRequireDefault(require("chalk"));

var _fclone = _interopRequireDefault(require("fclone"));

var _semver = _interopRequireDefault(require("semver"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _yamljs = _interopRequireDefault(require("yamljs"));

var _vm = _interopRequireDefault(require("vm"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _child_process = require("child_process");

var _cron = require("cron");

var _isbinaryfile = _interopRequireDefault(require("./tools/isbinaryfile"));

var _constants = _interopRequireDefault(require("./constants"));

var _interpreter = _interopRequireDefault(require("./API/interpreter"));

var _Config = _interopRequireDefault(require("./tools/Config"));

var _package = _interopRequireDefault(require("../package.json"));

var _which = _interopRequireDefault(require("./tools/which"));

var _passwd = _interopRequireDefault(require("./tools/passwd"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function homedir() {
  var env = process.env;
  var home = env.HOME;
  var user = env.LOGNAME || env.USER || env.LNAME || env.USERNAME;

  if (process.platform === "win32") {
    return env.USERPROFILE || env.HOMEDRIVE + env.HOMEPATH || home || null;
  }

  if (process.platform === "darwin") {
    return home || (user ? "/Users/" + user : null);
  }

  if (process.platform === "linux") {
    return home || (process.getuid() === 0 ? "/root" : user ? "/home/" + user : null);
  }

  return home || null;
}

function resolveHome(filepath) {
  if (filepath[0] === "~") {
    return _path["default"].join(homedir(), filepath.slice(1));
  }

  return filepath;
}
/**
 * Show warnings
 * @param {String} warning
 */


function warn(warning) {
  Common.printOut(_constants["default"].PREFIX_MSG_WARNING + warning);
}

var resolveNodeInterpreter = function resolveNodeInterpreter(app) {
  if (app.exec_mode && app.exec_mode.indexOf("cluster") > -1) {
    Common.printError(_constants["default"].PREFIX_MSG_WARNING + _chalk["default"].bold.yellow("Choosing the Node.js version in cluster mode is not supported"));
    return false;
  }

  var nvm_path = _constants["default"].IS_WINDOWS ? process.env.NVM_HOME : process.env.NVM_DIR;

  if (!nvm_path) {
    // Common.printError(cst.PREFIX_MSG_ERR + chalk.red("NVM is not available in PATH"));
    Common.printError(new Error(_constants["default"].PREFIX_MSG_ERR + _chalk["default"].red("NVM is not available in PATH")).stack); // Common.printError(cst.PREFIX_MSG_ERR + chalk.red("Fallback to node in PATH"));

    Common.printError(new Error(_constants["default"].PREFIX_MSG_ERR + _chalk["default"].red("Fallback to node in PATH")).stack);
    var msg = _constants["default"].IS_WINDOWS ? "https://github.com/coreybutler/nvm-windows/releases/" : "$ curl https://raw.githubusercontent.com/creationix/nvm/master/install.sh | bash"; // Common.printOut(cst.PREFIX_MSG_ERR + chalk.bold("Install NVM:\n" + msg));

    Common.printOut(new Error(_constants["default"].PREFIX_MSG_ERR + _chalk["default"].bold("Install NVM:\n" + msg)).stack);
  } else {
    var node_version = app.exec_interpreter.split("@")[1];
    var path_to_node = _constants["default"].IS_WINDOWS ? "/v" + node_version + "/node.exe" : _semver["default"].satisfies(node_version, ">= 0.12.0") ? "/versions/node/v" + node_version + "/bin/node" : "/v" + node_version + "/bin/node";

    var nvm_node_path = _path["default"].join(nvm_path, path_to_node);

    try {
      _fs["default"].accessSync(nvm_node_path);
    } catch (e) {
      Common.printOut(_constants["default"].PREFIX_MSG + "Installing Node v%s", node_version);

      var nvm_bin = _path["default"].join(nvm_path, "nvm." + (_constants["default"].IS_WINDOWS ? "exe" : "sh"));

      var nvm_cmd = _constants["default"].IS_WINDOWS ? nvm_bin + " install " + node_version : ". " + nvm_bin + " ; nvm install " + node_version;
      Common.printOut(_constants["default"].PREFIX_MSG + "Executing: %s", nvm_cmd);
      (0, _child_process.execSync)(nvm_cmd, {
        cwd: _path["default"].resolve(process.cwd()),
        env: process.env,
        maxBuffer: 20 * 1024 * 1024
      }); // in order to support both arch, nvm for Windows renames 'node.exe' to:
      // 'node32.exe' for x32 arch
      // 'node64.exe' for x64 arch

      if (_constants["default"].IS_WINDOWS) {
        nvm_node_path = nvm_node_path.replace(/node/, "node" + process.arch.slice(1));
      }
    }

    Common.printOut(_constants["default"].PREFIX_MSG + _chalk["default"].green.bold("Setting Node to v%s (path=%s)"), node_version, nvm_node_path);
    app.exec_interpreter = nvm_node_path;
  }
};

var Common = {
  determineSilentCLI: function determineSilentCLI() {
    // pm2 should ignore -s --silent -v if they are after '--'
    var variadicArgsDashesPos = process.argv.indexOf("--");
    var s1opt = process.argv.indexOf("--silent");
    var s2opt = process.argv.indexOf("-s");

    if (process.env.PM2_SILENT || variadicArgsDashesPos > -1 && s1opt != -1 && s1opt < variadicArgsDashesPos && s2opt != -1 != s2opt < variadicArgsDashesPos || variadicArgsDashesPos == -1 && (s1opt > -1 || s2opt > -1)) {
      for (var key in console) {
        var code = key.charCodeAt(0);

        if (code >= 97 && code <= 122) {
          console[key] = function () {// do nothing
          };
        }
      }

      process.env.PM2_DISCRETE_MODE = "true";
    }
  },
  printVersion: function printVersion() {
    var variadicArgsDashesPos = process.argv.indexOf("--");

    if (process.argv.indexOf("-v") > -1 && process.argv.indexOf("-v") < variadicArgsDashesPos) {
      console.log(_package["default"].version);
      process.exit(0);
    }
  },
  lockReload: function lockReload() {
    try {
      var t1 = _fs["default"].readFileSync(_constants["default"].PM2_RELOAD_LOCKFILE).toString(); // Check if content and if time < 30 return locked
      // Else if content detected (lock file staled), allow and rewritte


      if (t1 && t1 != "") {
        var diff = (0, _dayjs["default"])().diff(parseInt(t1));

        if (diff < _constants["default"].RELOAD_LOCK_TIMEOUT) {
          return diff;
        }
      }
    } catch (e) {}

    try {
      // Write latest timestamp
      _fs["default"].writeFileSync(_constants["default"].PM2_RELOAD_LOCKFILE, (0, _dayjs["default"])().valueOf().toString());

      return 0;
    } catch (e) {
      console.error(e.message || e);
    }
  },
  unlockReload: function unlockReload() {
    try {
      _fs["default"].writeFileSync(_constants["default"].PM2_RELOAD_LOCKFILE, "");
    } catch (e) {
      console.error(e.message || e);
    }
  },

  /**
   * Resolve app paths and replace missing values with defaults.
   * @method prepareAppConf
   * @param app {Object}
   * @param {} cwd
   * @param {} outputter
   * @return app
   */
  prepareAppConf: function prepareAppConf(opts, app) {
    /**
     * Minimum validation
     */
    if (!app.script) {
      return new Error("No script path - aborting");
    }

    var cwd = null;

    if (app.cwd) {
      cwd = _path["default"].resolve(app.cwd);
      process.env.PWD = app.cwd;
    }

    if (!app.node_args) {
      app.node_args = [];
    }

    if (app.port && app.env) {
      app.env.PORT = app.port;
    } // CWD option resolving


    cwd && cwd[0] != "/" && (cwd = _path["default"].resolve(process.cwd(), cwd));
    cwd = cwd || opts.cwd; // Full path script resolution

    app.pm_exec_path = _path["default"].resolve(cwd, app.script); // If script does not exist after resolution

    if (!_fs["default"].existsSync(app.pm_exec_path)) {
      var ckd; // Try resolve command available in $PATH

      if (ckd = (0, _which["default"])(app.script)) {
        if (typeof ckd !== "string") {
          ckd = ckd.toString();
        }

        app.pm_exec_path = ckd;
      } else {
        // Throw critical error
        return new Error("Script not found: ".concat(app.pm_exec_path));
      }
    }
    /**
     * Auto detect .map file and enable source map support automatically
     */


    if (app.disable_source_map_support != true) {
      try {
        _fs["default"].accessSync(app.pm_exec_path + ".map", _fs["default"].constants.R_OK);

        app.source_map_support = true;
      } catch (e) {}

      delete app.disable_source_map_support;
    }

    delete app.script; // Set current env by first adding the process environment and then extending/replacing it
    // with env specified on command-line or JSON file.

    var env = {};
    /**
     * Do not copy internal pm2 environment variables if acting on process
     * is made from a programmatic script started by PM2 or if a pm_id is present in env
     */

    if (_constants["default"].PM2_PROGRAMMATIC || process.env.pm_id) {
      Common.safeExtend(env, process.env);
    } else {
      env = process.env;
    }

    var filterEnv = function filterEnv(envObj) {
      if (app.filter_env == true) {
        return {};
      }

      if (typeof app.filter_env === "string") {
        delete envObj[app.filter_env];
        return envObj;
      }

      var new_env = {};
      var allowedKeys = app.filter_env.reduce(function (acc, current) {
        return acc.filter(function (item) {
          return !item.includes(current);
        });
      }, Object.keys(envObj));
      allowedKeys.forEach(function (key) {
        return new_env[key] = envObj[key];
      });
      return new_env;
    };

    app.env = [{}, app.filter_env && app.filter_env.length > 0 ? filterEnv(process.env) : env, app.env || {}].reduce(function (e1, e2) {
      e1 = _objectSpread(_objectSpread({}, e1), e2);
      return e1;
    });
    app.pm_cwd = cwd; // Interpreter

    try {
      Common.sink.resolveInterpreter(app);
    } catch (e) {
      return e;
    } // Exec mode and cluster stuff


    Common.sink.determineExecMode(app);
    /**
     * Scary
     */

    var formated_app_name = app.name.replace(/[^a-zA-Z0-9\\.\\-]/g, "-");
    ["log", "out", "error", "pid"].forEach(function (f) {
      var af = app[f + "_file"],
          ps;
      var isStd = !~["log", "pid"].indexOf(f);
      var ext = f == "pid" ? "pid" : "log";

      if (af) {
        af = resolveHome(af);
      }

      if (f == "log" && typeof af == "boolean" && af || f != "log" && !af) {
        ps = [_constants["default"]["DEFAULT_" + ext.toUpperCase() + "_PATH"], formated_app_name + (isStd ? "-" + f : "") + "." + ext];
      } else if ((f != "log" || f == "log" && af) && af !== "NULL" && af !== "/dev/null") {
        ps = [cwd, af];

        var dir = _path["default"].dirname(_path["default"].resolve(cwd, af));

        if (!_fs["default"].existsSync(dir)) {
          Common.printError(_constants["default"].PREFIX_MSG_WARNING + "Folder does not exist: " + dir);
          Common.printOut(_constants["default"].PREFIX_MSG + "Creating folder: " + dir);

          try {
            _mkdirp["default"].sync(dir);
          } catch (err) {
            // Common.printError(cst.PREFIX_MSG_ERR + "Could not create folder: " + path.dirname(af));
            Common.printError(new Error(_constants["default"].PREFIX_MSG_ERR + "Could not create folder: " + _path["default"].dirname(af)).stack);
            throw new Error("Could not create folder");
          }
        }
      } // PM2 paths


      if (af !== "NULL" && af !== "/dev/null") {
        ps && (app["pm_" + (isStd ? f.substr(0, 3) + "_" : "") + ext + "_path"] = _path["default"].resolve.apply(null, ps));
      } else if (_path["default"].sep === "\\") {
        app["pm_" + (isStd ? f.substr(0, 3) + "_" : "") + ext + "_path"] = "\\\\.\\NUL";
      } else {
        app["pm_" + (isStd ? f.substr(0, 3) + "_" : "") + ext + "_path"] = "/dev/null";
      }

      delete app[f + "_file"];
    });
    return app;
  },

  /**
   * Check if filename is a configuration file
   * @param {string} filename
   * @return {mixed} null if not conf file, json or yaml if conf
   */
  isConfigFile: function isConfigFile(filename) {
    if (typeof filename !== "string") {
      return null;
    }

    if (filename.indexOf(".json") !== -1) {
      return "json";
    }

    if (filename.indexOf(".yml") > -1 || filename.indexOf(".yaml") > -1) {
      return "yaml";
    }

    if (filename.indexOf(".config.js") !== -1) {
      return "js";
    }

    if (filename.indexOf(".config.cjs") !== -1) {
      return "js";
    }

    if (filename.indexOf(".config.mjs") !== -1) {
      return "mjs";
    }

    return null;
  },

  /**
   * Parses a config file like ecosystem.config.js. Supported formats: JS, JSON, JSON5, YAML.
   * @param {string} confString  contents of the config file
   * @param {string} filename    path to the config file
   * @return {Object} config object
   */
  parseConfig: function parseConfig(confObj, filename) {
    if (!filename || filename == "pipe" || filename == "none" || filename.indexOf(".json") > -1) {
      var code = "(" + confObj + ")"; // const sandbox = {};
      // TODO: Please check this
      // return vm.runInThisContext(code, sandbox, {
      //   filename: path.resolve(filename),
      //   displayErrors: false,
      //   timeout: 1000
      // });

      return _vm["default"].runInThisContext(code, {
        filename: _path["default"].resolve(filename),
        displayErrors: false,
        timeout: 1000
      });
    } else if (filename.indexOf(".yml") > -1 || filename.indexOf(".yaml") > -1) {
      return _yamljs["default"].parse(confObj.toString());
    } else if (filename.indexOf(".config.js") > -1 || filename.indexOf(".config.cjs") > -1 || filename.indexOf(".config.mjs") > -1) {
      var confPath = require.resolve(_path["default"].resolve(filename));

      delete require.cache[confPath];
      return require(confPath);
    }
  },
  retErr: function retErr(e) {
    if (!e) {
      return new Error("Unidentified error");
    }

    if (e instanceof Error) {
      return e;
    }

    return new Error(e);
  },
  sink: {
    determineCron: function determineCron(app) {
      if (app.cron_restart) {
        try {
          Common.printOut(_constants["default"].PREFIX_MSG + "cron restart at " + app.cron_restart);
          new _cron.CronJob(app.cron_restart, function () {
            Common.printOut(_constants["default"].PREFIX_MSG + "cron pattern for auto restart detected and valid");
          });
        } catch (ex) {
          return new Error("Cron pattern error: ".concat(ex.message));
        }
      }
    },

    /**
     * Handle alias (fork <=> fork_mode, cluster <=> cluster_mode)
     */
    determineExecMode: function determineExecMode(app) {
      if (app.exec_mode) {
        app.exec_mode = app.exec_mode.replace(/^(fork|cluster)$/, "$1_mode");
      }
      /**
       * Here we put the default exec mode
       */


      if (!app.exec_mode && (app.instances >= 1 || app.instances === 0 || app.instances === -1) && app.exec_interpreter.indexOf("node") > -1) {
        app.exec_mode = "cluster_mode";
      } else if (!app.exec_mode) {
        app.exec_mode = "fork_mode";
      }

      if (typeof app.instances == "undefined") {
        app.instances = 1;
      }
    },

    /**
     * Resolve interpreter
     */
    resolveInterpreter: function resolveInterpreter(app) {
      var noInterpreter = !app.exec_interpreter;

      var extName = _path["default"].extname(app.pm_exec_path);

      var betterInterpreter = _interpreter["default"][extName]; // No interpreter defined and correspondance in schema hashmap

      if (noInterpreter && betterInterpreter) {
        app.exec_interpreter = betterInterpreter;
      } else if (noInterpreter) {
        // Else if no Interpreter detect if process is binary
        app.exec_interpreter = (0, _isbinaryfile["default"])(app.pm_exec_path) ? "none" : "node";
      } else if (app.exec_interpreter.indexOf("node@") > -1) {
        resolveNodeInterpreter(app);
      }

      if (app.exec_interpreter.indexOf("python") > -1) {
        app.env.PYTHONUNBUFFERED = "1";
      }
      /**
       * Specific installed JS transpilers
       */


      if (app.exec_interpreter == "ts-node") {
        app.exec_interpreter = _path["default"].resolve(__dirname, "../node_modules/.bin/ts-node");
      }

      if (app.exec_interpreter == "lsc") {
        app.exec_interpreter = _path["default"].resolve(__dirname, "../node_modules/.bin/lsc");
      }

      if (app.exec_interpreter == "coffee") {
        app.exec_interpreter = _path["default"].resolve(__dirname, "../node_modules/.bin/coffee");
      }

      if (app.exec_interpreter != "none" && (0, _which["default"])(app.exec_interpreter) == null) {
        // If node is not present
        if (app.exec_interpreter == "node") {
          Common.warn("Using builtin node.js version on version ".concat(process.version));
          app.exec_interpreter = _constants["default"].BUILTIN_NODE_PATH;
        } else {
          throw new Error("Interpreter ".concat(app.exec_interpreter, " is NOT AVAILABLE in PATH. (type 'which ").concat(app.exec_interpreter, "' to double check.)"));
        }
      }

      return app;
    }
  },
  deepCopy: function deepCopy(obj) {
    if (obj === null || obj === undefined) {
      return {};
    }

    return (0, _fclone["default"])(obj);
  },
  serialize: function serialize(obj) {
    if (obj === null || obj === undefined) {
      return {};
    }

    return (0, _fclone["default"])(obj);
  },
  clone: function clone(obj) {
    if (obj === null || obj === undefined) {
      return {};
    }

    return (0, _fclone["default"])(obj);
  },
  errMod: function errMod(msg) {
    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    if (msg instanceof Error) {
      return console.error(msg.message);
    }

    return console.error("".concat(_constants["default"].PREFIX_MSG_MOD_ERR).concat(msg));
  },
  err: function err(msg) {
    console.log("Error 1");

    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    if (msg instanceof Error) {
      console.log("Error 11"); // return console.error(`${cst.PREFIX_MSG_ERR}${msg.message}`);

      return console.error("".concat(_constants["default"].PREFIX_MSG_ERR).concat(msg.stack));
    }

    console.trace("Error 12");
    return console.error("".concat(_constants["default"].PREFIX_MSG_ERR).concat(msg));
  },
  printError: function printError() {
    var _console;

    console.log("Error 2");

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var msg = args[0];

    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    if (msg instanceof Error) {
      console.log("Error 21"); // return console.error(msg.message);

      return console.error(msg.stack);
    }

    console.log("Error 22"); // return console.error(...args);

    return (_console = console).error.apply(_console, args);
  },
  log: function log(msg) {
    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    return console.log("".concat(_constants["default"].PREFIX_MSG).concat(msg));
  },
  info: function info(msg) {
    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    return console.log("".concat(_constants["default"].PREFIX_MSG_INFO).concat(msg));
  },
  warn: function warn(msg) {
    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    return console.log("".concat(_constants["default"].PREFIX_MSG_WARNING).concat(msg));
  },
  logMod: function logMod(msg) {
    if (process.env.PM2_SILENT || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    }

    return console.log("".concat(_constants["default"].PREFIX_MSG_MOD).concat(msg));
  },
  printOut: function printOut() {
    var _console2;

    if (process.env.PM2_SILENT === "true" || process.env.PM2_PROGRAMMATIC === "true") {
      return false;
    } // return console.log.apply(console, arguments);


    return (_console2 = console).log.apply(_console2, arguments);
  },

  /**
   * Raw extend
   */
  extend: function extend(destination, source) {
    if ((0, _typeof2["default"])(destination) !== "object") {
      destination = {};
    }

    if (!source || (0, _typeof2["default"])(source) !== "object") {
      return destination;
    }

    Object.keys(source).forEach(function (new_key) {
      if (source[new_key] != "[object Object]") {
        destination[new_key] = source[new_key];
      }
    });
    return destination;
  },

  /**
   * This is useful when starting script programmatically
   */
  safeExtend: function safeExtend(origin, add) {
    if (!add || (0, _typeof2["default"])(add) != "object") {
      return origin;
    } //Ignore PM2's set environment variables from the nested env


    var keysToIgnore = ["name", "exec_mode", "env", "args", "pm_cwd", "exec_interpreter", "pm_exec_path", "node_args", "pm_out_log_path", "pm_err_log_path", "pm_pid_path", "pm_id", "status", "pm_uptime", "created_at", "windowsHide", "username", "merge_logs", "kill_retry_time", "prev_restart_delay", "instance_var", "unstable_restarts", "restart_time", "axm_actions", "pmx_module", "command", "watch", "filter_env", "versioning", "vizion_runing", "MODULE_DEBUG", "pmx", "axm_options", "created_at", "watch", "vizion", "axm_dynamic", "axm_monitor", "instances", "automation", "autorestart", "unstable_restart", "treekill", "exit_code", "vizion"];
    var keys = Object.keys(add);
    var i = keys.length;

    while (i--) {
      //Only copy stuff into the env that we don't have already.
      if (keysToIgnore.indexOf(keys[i]) == -1 && add[keys[i]] != "[object Object]") {
        origin[keys[i]] = add[keys[i]];
      }
    }

    return origin;
  },

  /**
   * Extend the app.env object of with the properties taken from the
   * app.env_[envName] and deploy configuration.
   * Also update current json attributes
   *
   * Used only for Configuration file processing
   *
   * @param {Object} app The app object.
   * @param {string} envName The given environment name.
   * @param {Object} deployConf Deployment configuration object (from JSON file or whatever).
   * @returns {Object} The app.env variables object.
   */
  mergeEnvironmentVariables: function mergeEnvironmentVariables(app_env, env_name, deploy_conf) {
    var app = (0, _fclone["default"])(app_env);
    var new_conf = {
      env: {}
    }; // Stringify possible object

    for (var key in app.env) {
      if ((0, _typeof2["default"])(app.env[key]) == "object") {
        app.env[key] = JSON.stringify(app.env[key]);
      }
    }
    /**
     * Extra configuration update
     */


    new_conf = _objectSpread(_objectSpread({}, new_conf), app);

    if (env_name) {
      // First merge variables from deploy.production.env object as least priority.
      if (deploy_conf && deploy_conf[env_name] && deploy_conf[env_name]["env"]) {
        new_conf.env = _objectSpread(_objectSpread({}, new_conf.env), deploy_conf[env_name]["env"]);
      }

      new_conf.env = _objectSpread(_objectSpread({}, new_conf.env), app.env); // Then, last and highest priority, merge the app.env_production object.

      if ("env_" + env_name in app) {
        new_conf.env = _objectSpread(_objectSpread({}, new_conf.env), app["env_" + env_name]);
      } else {
        Common.printOut(_constants["default"].PREFIX_MSG_WARNING + _chalk["default"].bold("Environment [%s] is not defined in process file"), env_name);
      }
    }

    delete new_conf.exec_mode;
    var res = {
      current_conf: {}
    };
    res = _objectSpread(_objectSpread({}, res), new_conf.env);
    res.current_conf = _objectSpread(_objectSpread({}, res.current_conf), new_conf); // #2541 force resolution of node interpreter

    if (app.exec_interpreter && app.exec_interpreter.indexOf("@") > -1) {
      resolveNodeInterpreter(app);
      res.current_conf.exec_interpreter = app.exec_interpreter;
    }

    return res;
  },

  /**
   * This function will resolve paths, option and environment
   * CALLED before 'prepare' God call (=> PROCESS INITIALIZATION)
   * @method resolveAppAttributes
   * @param {Object} opts
   * @param {Object} opts.cwd
   * @param {Object} opts.pm2_home
   * @param {Object} appConf application configuration
   * @return app
   */
  resolveAppAttributes: function resolveAppAttributes(opts, conf) {
    console.log("conf", conf);
    var conf_copy = (0, _fclone["default"])(conf);
    console.log("conf_copy", conf_copy);
    var app = Common.prepareAppConf(opts, conf_copy);

    if (app instanceof Error) {
      throw new Error(app.message);
    }

    return app;
  },

  /**
   * Verify configurations
   * Called on EVERY Operation (start/restart/reload/stop...)
   * @param {Array} appConfs
   * @returns {Array}
   */
  verifyConfs: function verifyConfs(appConfs) {
    if (!appConfs || appConfs.length == 0) {
      return [];
    } // Make sure it is an Array.


    appConfs = [].concat(appConfs);
    var verifiedConf = [];

    for (var i = 0; i < appConfs.length; i++) {
      var app = appConfs[i];

      if (app.exec_mode) {
        app.exec_mode = app.exec_mode.replace(/^(fork|cluster)$/, "$1_mode");
      } // JSON conf: alias cmd to script


      if (app.cmd && !app.script) {
        app.script = app.cmd;
        delete app.cmd;
      } // JSON conf: alias command to script


      if (app.command && !app.script) {
        app.script = app.command;
        delete app.command;
      }

      if (!app.env) {
        app.env = {};
      } // Render an app name if not existing.


      Common.renderApplicationName(app);

      if (app.execute_command == true) {
        app.exec_mode = "fork";
        delete app.execute_command;
      }

      app.username = Common.getCurrentUsername();
      /**
       * If command is like pm2 start "python xx.py --ok"
       * Then automatically start the script with bash -c and set a name eq to command
       */

      if (app.script && app.script.indexOf(" ") > -1 && _constants["default"].IS_WINDOWS === false) {
        var _script = app.script;

        if ((0, _which["default"])("bash")) {
          app.script = "bash";
          app.args = ["-c", _script];

          if (!app.name) {
            app.name = _script;
          }
        } else if ((0, _which["default"])("sh")) {
          app.script = "sh";
          app.args = ["-c", _script];

          if (!app.name) {
            app.name = _script;
          }
        } else {
          warn("bash or sh not available in $PATH, keeping script as is");
        }
      }
      /**
       * Add log_date_format by default
       */


      if (app.time) {
        app.log_date_format = "YYYY-MM-DDTHH:mm:ss";
      }
      /**
       * Checks + Resolve UID/GID
       * comes from pm2 --uid <> --gid <> or --user
       */


      if (app.uid || app.gid || app.user) {
        // 1/ Check if windows
        if (_constants["default"].IS_WINDOWS === true) {
          // Common.printError(cst.PREFIX_MSG_ERR + "--uid and --git does not works on windows");
          Common.printError(new Error(_constants["default"].PREFIX_MSG_ERR + "--uid and --git does not works on windows").stack);
          return new Error("--uid and --git does not works on windows");
        } // 2/ Verify that user is root (todo: verify if other has right)


        if (process.env.NODE_ENV != "test" && process.getuid && process.getuid() !== 0) {
          // Common.printError(cst.PREFIX_MSG_ERR + "To use --uid and --gid please run pm2 as root");
          Common.printError(new Error(_constants["default"].PREFIX_MSG_ERR + "To use --uid and --gid please run pm2 as root").stack);
          return new Error("To use UID and GID please run PM2 as root");
        } // 3/ Resolve user info via /etc/password


        var users = void 0;

        try {
          users = _passwd["default"].getUsers();
        } catch (e) {
          Common.printError(e);
          return new Error(e);
        }

        var user_info = users[app.uid || app.user];

        if (!user_info) {
          // Common.printError(`${cst.PREFIX_MSG_ERR} User ${app.uid || app.user} cannot be found`);
          Common.printError(new Error("".concat(_constants["default"].PREFIX_MSG_ERR, " User ").concat(app.uid || app.user, " cannot be found")).stack);
          return new Error("".concat(_constants["default"].PREFIX_MSG_ERR, " User ").concat(app.uid || app.user, " cannot be found"));
        }

        app.env.HOME = user_info.homedir;
        app.uid = parseInt(user_info.userId); // 4/ Resolve group id if gid is specified

        if (app.gid) {
          var groups = void 0;

          try {
            groups = _passwd["default"].getGroups();
          } catch (e) {
            Common.printError(e);
            return new Error(e);
          }

          var group_info = groups[app.gid];

          if (!group_info) {
            Common.printError("".concat(_constants["default"].PREFIX_MSG_ERR, " Group ").concat(app.gid, " cannot be found"));
            Common.printError(new Error("".concat(_constants["default"].PREFIX_MSG_ERR, " Group ").concat(app.gid, " cannot be found")).stack);
            return new Error("".concat(_constants["default"].PREFIX_MSG_ERR, " Group ").concat(app.gid, " cannot be found"));
          }

          app.gid = parseInt(group_info.id);
        } else {
          app.gid = parseInt(user_info.groupId);
        }
      }
      /**
       * Specific options of PM2.io
       */


      if (process.env.PM2_DEEP_MONITORING) {
        app.deep_monitoring = true;
      }

      if (app.automation == false) {
        app.pmx = false;
      }

      if (app.disable_trace) {
        app.trace = false;
        delete app.disable_trace;
      }
      /**
       * Instances params
       */


      if (app.instances == "max") {
        app.instances = 0;
      }

      if (typeof app.instances === "string") {
        app.instances = parseInt(app.instances) || 0;
      }

      if (app.exec_mode != "cluster_mode" && !app.instances && typeof app.merge_logs == "undefined") {
        app.merge_logs = true;
      }

      var ret = void 0;

      if (app.cron_restart) {
        if ((ret = Common.sink.determineCron(app)) instanceof Error) {
          return ret;
        }
      }
      /**
       * Now validation configuration
       */


      ret = _Config["default"].validateJSON(app);

      if (ret.errors && ret.errors.length > 0) {
        ret.errors.forEach(function (err) {
          warn(err);
        });
        return new Error(ret.errors);
      }

      verifiedConf.push(ret.config);
    }

    return verifiedConf;
  },

  /**
   * Get current username
   * Called on EVERY starting app
   *
   * @returns {String}
   */
  getCurrentUsername: function getCurrentUsername() {
    var current_user = "";

    if (_os["default"].userInfo) {
      try {
        current_user = _os["default"].userInfo().username;
      } catch (err) {// For the case of unhandled error for uv_os_get_passwd
        // https://github.com/Unitech/pm2/issues/3184
      }
    }

    if (current_user === "") {
      current_user = process.env.USER || process.env.LNAME || process.env.USERNAME || process.env.SUDO_USER || process.env.C9_USER || process.env.LOGNAME;
    }

    return current_user;
  },

  /**
   * Render an app name if not existing.
   * @param {Object} conf
   */
  renderApplicationName: function renderApplicationName(conf) {
    if (!conf.name && conf.script) {
      conf.name = conf.script !== undefined ? _path["default"].basename(conf.script) : "undefined";
      var lastDot = conf.name.lastIndexOf(".");

      if (lastDot > 0) {
        conf.name = conf.name.slice(0, lastDot);
      }
    }
  }
};
var _default = Common;
exports["default"] = _default;