"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _fs = _interopRequireDefault(require("fs"));

var _inspector = _interopRequireDefault(require("inspector"));

var _debug = _interopRequireDefault(require("debug"));

var _semver = _interopRequireDefault(require("semver"));

var _domain = _interopRequireDefault(require("domain"));

var _pm2Axon = _interopRequireDefault(require("pm2-axon"));

var _pm2AxonRpc = _interopRequireDefault(require("pm2-axon-rpc"));

var _child_process = require("child_process");

var _package = _interopRequireDefault(require("../package.json"));

var _constants = _interopRequireDefault(require("./constants"));

var _Utility = _interopRequireDefault(require("./Utility"));

var _God = _interopRequireDefault(require("./God"));

var _eachLimit = _interopRequireDefault(require("async/eachLimit"));

var fmt = _interopRequireWildcard(require("./tools/fmt"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Copyright 2013 the PM2 project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */
var debug = (0, _debug["default"])("pm2:daemon");

var Daemon = function Daemon(_opts) {
  var _this = this;

  (0, _classCallCheck2["default"])(this, Daemon);
  (0, _defineProperty2["default"])(this, "ignore_signals", void 0);
  (0, _defineProperty2["default"])(this, "rpc_socket_ready", void 0);
  (0, _defineProperty2["default"])(this, "pub_socket_ready", void 0);
  (0, _defineProperty2["default"])(this, "pub_socket_file", void 0);
  (0, _defineProperty2["default"])(this, "rpc_socket_file", void 0);
  (0, _defineProperty2["default"])(this, "pid_path", void 0);
  (0, _defineProperty2["default"])(this, "pub", void 0);
  (0, _defineProperty2["default"])(this, "pub_socket", void 0);
  (0, _defineProperty2["default"])(this, "rep", void 0);
  (0, _defineProperty2["default"])(this, "rpc_socket", void 0);
  (0, _defineProperty2["default"])(this, "isExiting", void 0);
  (0, _defineProperty2["default"])(this, "start", function () {
    var d = _domain["default"].create();

    d.once("error", function (err) {
      fmt.sep();
      fmt.title("PM2 global error caught");
      fmt.field("Time", new Date());
      console.error(err.message);
      console.error(err.stack);
      fmt.sep();
      console.error("[PM2] Resurrecting PM2");
      var path = _constants["default"].IS_WINDOWS ? __dirname + "/../pm2t" : process.env["_"];
      var fork_new_pm2 = (0, _child_process.spawn)("node", [path, "update"], {
        detached: true,
        stdio: "inherit"
      });
      fork_new_pm2.on("close", function () {
        console.log("PM2 successfully forked");
        process.exit(0);
      });
    });
    d.run(function () {
      _this.innerStart();
    });
  });
  (0, _defineProperty2["default"])(this, "innerStart", function (cb) {
    if (!cb) {
      cb = function cb() {
        fmt.sep();
        fmt.title("New PM2 Daemon started");
        fmt.field("Time", new Date());
        fmt.field("PM2 version", _package["default"].version);
        fmt.field("Node.js version", process.versions.node);
        fmt.field("Current arch", process.arch);
        fmt.field("PM2 home", _constants["default"].PM2_HOME);
        fmt.field("PM2 PID file", _this.pid_path);
        fmt.field("RPC socket file", _this.rpc_socket_file);
        fmt.field("BUS socket file", _this.pub_socket_file);
        fmt.field("Application log path", _constants["default"].DEFAULT_LOG_PATH);
        fmt.field("Worker Interval", _constants["default"].WORKER_INTERVAL);
        fmt.field("Process dump file", _constants["default"].DUMP_FILE_PATH);
        fmt.field("Concurrent actions", _constants["default"].CONCURRENT_ACTIONS);
        fmt.field("SIGTERM timeout", _constants["default"].KILL_TIMEOUT);
        fmt.sep();
      };
    } // Write Daemon PID into file


    try {
      _fs["default"].writeFileSync(_this.pid_path, process.pid.toString());
    } catch (e) {
      console.error(e.stack || e);
    }

    if (_this.ignore_signals != true) {
      _this.handleSignals();
    }
    /**
     * Pub system for real time notifications
     */


    _this.pub = _pm2Axon["default"].socket("pub-emitter");
    _this.pub_socket = _this.pub.bind(_this.pub_socket_file);

    _this.pub_socket.once("bind", function () {
      _fs["default"].chmod(_this.pub_socket_file, "775", function (e) {
        if (e) {
          console.error(e);
        }

        try {
          if (process.env.PM2_SOCKET_USER && process.env.PM2_SOCKET_GROUP) {
            _fs["default"].chown(_this.pub_socket_file, parseInt(process.env.PM2_SOCKET_USER), parseInt(process.env.PM2_SOCKET_GROUP), function (e) {
              if (e) {
                console.error(e);
              }
            });
          }
        } catch (e) {
          console.error(e);
        }
      });

      _this.pub_socket_ready = true;

      _this.sendReady(cb);
    });
    /**
     * Rep/Req - RPC system to interact with God
     */


    _this.rep = _pm2Axon["default"].socket("rep");
    var server = new _pm2AxonRpc["default"].Server(_this.rep);
    _this.rpc_socket = _this.rep.bind(_this.rpc_socket_file);

    _this.rpc_socket.once("bind", function () {
      _fs["default"].chmod(_this.rpc_socket_file, "775", function (e) {
        if (e) {
          console.error(e);
        }

        try {
          if (process.env.PM2_SOCKET_USER && process.env.PM2_SOCKET_GROUP) {
            _fs["default"].chown(_this.rpc_socket_file, parseInt(process.env.PM2_SOCKET_USER), parseInt(process.env.PM2_SOCKET_GROUP), function (e) {
              if (e) {
                console.error(e);
              }
            });
          }
        } catch (e) {
          console.error(e);
        }
      });

      _this.rpc_socket_ready = true;

      _this.sendReady(cb);
    });
    /**
     * Memory Snapshot
     */


    function profile(type, msg, cb) {
      if (_semver["default"].satisfies(process.version, "< 8")) {
        return cb(null, {
          error: "Node.js is not on right version"
        });
      }

      var cmd;

      if (type === "cpu") {
        cmd = {
          enable: "Profiler.enable",
          start: "Profiler.start",
          stop: "Profiler.stop",
          disable: "Profiler.disable"
        };
      }

      if (type == "mem") {
        cmd = {
          enable: "HeapProfiler.enable",
          start: "HeapProfiler.startSampling",
          stop: "HeapProfiler.stopSampling",
          disable: "HeapProfiler.disable"
        };
      }

      var session = new _inspector["default"].Session();
      session.connect();
      var timeout = msg.timeout || 5000;
      session.post(cmd.enable, function (err, data) {
        if (err) {
          return cb(null, {
            error: err.message || err
          });
        }

        console.log("Starting ".concat(cmd.start));
        session.post(cmd.start, function (err, data) {
          if (err) {
            return cb(null, {
              error: err.message || err
            });
          }

          setTimeout(function () {
            session.post(cmd.stop, function (err, data) {
              if (err) {
                return cb(null, {
                  error: err.message || err
                });
              }

              var profile = data.profile;
              console.log("Stopping ".concat(cmd.stop));
              session.post(cmd.disable);

              _fs["default"].writeFile(msg.pwd, JSON.stringify(profile), function (err) {
                if (err) {
                  return cb(null, {
                    error: err.message || err
                  });
                }

                return cb(null, {
                  file: msg.pwd
                });
              });
            });
          }, timeout);
        });
      });
    }

    server.expose({
      killMe: _this.close.bind(_this),
      profileCPU: profile.bind(_this, "cpu"),
      profileMEM: profile.bind(_this, "mem"),
      prepare: _God["default"].prepare,
      launchSysMonitoring: _God["default"].launchSysMonitoring,
      getMonitorData: _God["default"].getMonitorData,
      getSystemData: _God["default"].getSystemData,
      startProcessId: _God["default"].startProcessId,
      stopProcessId: _God["default"].stopProcessId,
      restartProcessId: _God["default"].restartProcessId,
      deleteProcessId: _God["default"].deleteProcessId,
      sendLineToStdin: _God["default"].sendLineToStdin,
      softReloadProcessId: _God["default"].softReloadProcessId,
      reloadProcessId: _God["default"].reloadProcessId,
      duplicateProcessId: _God["default"].duplicateProcessId,
      resetMetaProcessId: _God["default"].resetMetaProcessId,
      stopWatch: _God["default"].stopWatch,
      startWatch: _God["default"].startWatch,
      toggleWatch: _God["default"].toggleWatch,
      notifyByProcessId: _God["default"].notifyByProcessId,
      notifyKillPM2: _God["default"].notifyKillPM2,
      monitor: _God["default"].monitor,
      unmonitor: _God["default"].unmonitor,
      msgProcess: _God["default"].msgProcess,
      sendDataToProcessId: _God["default"].sendDataToProcessId,
      sendSignalToProcessId: _God["default"].sendSignalToProcessId,
      sendSignalToProcessName: _God["default"].sendSignalToProcessName,
      ping: _God["default"].ping,
      getVersion: _God["default"].getVersion,
      getReport: _God["default"].getReport,
      reloadLogs: _God["default"].reloadLogs
    });

    _this.startLogic();
  });
  (0, _defineProperty2["default"])(this, "close", function (opts, cb) {
    _God["default"].bus.emit("pm2:kill", {
      status: "killed",
      msg: "pm2 has been killed via CLI"
    });

    if (_God["default"].system_infos_proc !== null) {
      _God["default"].system_infos_proc.kill();
    }
    /**
     * Cleanly kill pm2
     */


    _this.rpc_socket.close(function () {
      _this.pub_socket.close(function () {
        // notify cli that the daemon is shuting down (only under unix since windows doesnt handle signals)
        if (_constants["default"].IS_WINDOWS === false) {
          try {
            process.kill(parseInt(opts.pid), "SIGQUIT");
          } catch (e) {
            console.error("Could not send SIGQUIT to CLI");
          }
        }

        try {
          _fs["default"].unlinkSync(_this.pid_path);
        } catch (e) {// do nothing
        }

        console.log("PM2 successfully stopped");
        setTimeout(function () {
          process.exit(_constants["default"].SUCCESS_EXIT);
        }, 2);
      });
    });
  });
  (0, _defineProperty2["default"])(this, "handleSignals", function () {
    process.on("SIGTERM", _this.gracefullExit.bind(_this));
    process.on("SIGINT", _this.gracefullExit.bind(_this));
    process.on("SIGHUP", function () {// do nothing
    });
    process.on("SIGQUIT", _this.gracefullExit.bind(_this));
    process.on("SIGUSR2", function () {
      _God["default"].reloadLogs({}, function () {// do nothing
      });
    });
  });
  (0, _defineProperty2["default"])(this, "sendReady", function (cb) {
    // Send ready message to Client
    if (_this.rpc_socket_ready == true && _this.pub_socket_ready == true) {
      cb(null, {
        pid: process.pid,
        pm2_version: _package["default"].version
      });

      if (typeof process.send != "function") {
        return false;
      }

      process.send({
        online: true,
        success: true,
        pid: process.pid,
        pm2_version: _package["default"].version
      });
    }
  });
  (0, _defineProperty2["default"])(this, "gracefullExit", function () {
    // never execute multiple gracefullExit simultaneously
    // this can lead to loss of some apps in dump file
    if (_this.isExiting) {
      return;
    }

    _this.isExiting = true;

    _God["default"].bus.emit("pm2:kill", {
      status: "killed",
      msg: "pm2 has been killed by SIGNAL"
    });

    console.log("pm2 has been killed by signal, dumping process list before exit...");

    if (_God["default"].system_infos_proc !== null) {
      _God["default"].system_infos_proc.kill();
    }

    _God["default"].dumpProcessList(function () {
      var processes = _God["default"].getFormatedProcesses();

      (0, _eachLimit["default"])(processes, 1, function (proc, next) {
        console.log("Deleting process %s", proc.pm2_env.pm_id);

        _God["default"].deleteProcessId(proc.pm2_env.pm_id, function () {
          return next();
        });
      }, function (err) {
        try {
          _fs["default"].unlinkSync(_this.pid_path);
        } catch (e) {// do nothing
        }

        setTimeout(function () {
          _this.isExiting = false;
          console.log("Exited peacefully");
          process.exit(_constants["default"].SUCCESS_EXIT);
        }, 2);
      });
    });
  });
  (0, _defineProperty2["default"])(this, "startLogic", function () {
    /**
     * Action treatment specifics
     * Attach actions to pm2_env.axm_actions variables (name + options)
     */
    _God["default"].bus.on("axm:action", function axmActions(msg) {
      var pm2_env = msg.process;
      var exists = false;
      var axm_action = msg.data;

      if (!pm2_env || !_God["default"].clusters_db[pm2_env.pm_id]) {
        return console.error("AXM ACTION Unknown id %s", pm2_env.pm_id);
      }

      if (!_God["default"].clusters_db[pm2_env.pm_id].pm2_env.axm_actions) {
        _God["default"].clusters_db[pm2_env.pm_id].pm2_env.axm_actions = [];
      }

      _God["default"].clusters_db[pm2_env.pm_id].pm2_env.axm_actions.forEach(function (actions) {
        if (actions.action_name == axm_action.action_name) {
          exists = true;
        }
      });

      if (exists === false) {
        debug("Adding action", axm_action);

        _God["default"].clusters_db[pm2_env.pm_id].pm2_env.axm_actions.push(axm_action);
      }

      msg = null;
    });
    /**
     * Configure module
     */


    _God["default"].bus.on("axm:option:configuration", function (msg) {
      if (!msg.process) {
        return console.error("[axm:option:configuration] no process defined");
      }

      if (!_God["default"].clusters_db[msg.process.pm_id]) {
        return console.error("[axm:option:configuration] Unknown id %s", msg.process.pm_id);
      }

      try {
        // Application Name nverride
        if (msg.data.name) {
          _God["default"].clusters_db[msg.process.pm_id].pm2_env.name = msg.data.name;
        }

        Object.keys(msg.data).forEach(function (conf_key) {
          _God["default"].clusters_db[msg.process.pm_id].pm2_env.axm_options[conf_key] = _Utility["default"].clone(msg.data[conf_key]);
        });
      } catch (e) {
        console.error(e.stack || e);
      }

      msg = null;
    });
    /**
     * Process monitoring data (probes)
     */


    _God["default"].bus.on("axm:monitor", function axmMonitor(msg) {
      if (!msg.process) {
        return console.error("[axm:monitor] no process defined");
      }

      if (!msg.process || !_God["default"].clusters_db[msg.process.pm_id]) {
        return console.error("AXM MONITOR Unknown id %s", msg.process.pm_id);
      }

      _God["default"].clusters_db[msg.process.pm_id].pm2_env.axm_monitor = Object.assign({}, _God["default"].clusters_db[msg.process.pm_id].pm2_env.axm_monitor, _Utility["default"].clone(msg.data));
      msg = null;
    });
    /**
     * Broadcast messages
     */


    _God["default"].bus.onAny(function (event, data_v) {
      if (typeof event === "string") {
        if (["axm:action", "axm:monitor", "axm:option:setPID", "axm:option:configuration"].indexOf(event) > -1) {
          data_v = null;
          return false;
        }
      }

      _this.pub.emit(event, _Utility["default"].clone(data_v));

      data_v = null;
    });
  });

  if (!_opts) {
    _opts = {};
  }

  this.ignore_signals = _opts.ignore_signals || false;
  this.rpc_socket_ready = false;
  this.pub_socket_ready = false;
  this.pub_socket_file = _opts.pub_socket_file || _constants["default"].DAEMON_PUB_PORT;
  this.rpc_socket_file = _opts.rpc_socket_file || _constants["default"].DAEMON_RPC_PORT;
  this.pid_path = _opts.pid_file || _constants["default"].PM2_PID_FILE_PATH;
};

if (require.main === module) {
  process.title = process.env.PM2_DAEMON_TITLE || "PM2 v" + _package["default"].version + ": God Daemon (" + process.env.PM2_HOME + ")";
  var daemon = new Daemon();
  daemon.start();
}

var _default = Daemon;
exports["default"] = _default;