"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _http = _interopRequireDefault(require("http"));

var _os = _interopRequireDefault(require("os"));

var _index = _interopRequireDefault(require("../index.js"));

var _url = _interopRequireDefault(require("url"));

var _constants = _interopRequireDefault(require("./constants"));

/**
 * Copyright 2013 the PM2 project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */
// Default, attach to default local PM2
_index["default"].connect(function () {
  startWebServer(_index["default"]);
});

function startWebServer(pm2) {
  _http["default"].createServer(function (req, res) {
    // Add CORS headers to allow browsers to fetch data directly
    res.setHeader("Access-Control-Allow-Origin", "*");
    res.setHeader("Access-Control-Allow-Headers", "Cache-Control, Pragma, Origin, Authorization, Content-Type, X-Requested-With");
    res.setHeader("Access-Control-Allow-Methods", "GET"); // We always send json

    res.setHeader("Content-Type", "application/json");

    var path = _url["default"].parse(req.url).pathname;

    if (path == "/") {
      // Main monit route
      pm2.list(function (err, list) {
        if (err) {
          // return res.send(err);
          return res.end(err);
        }

        var data = {
          system_info: {
            hostname: _os["default"].hostname(),
            uptime: _os["default"].uptime()
          },
          monit: {
            loadavg: _os["default"].loadavg(),
            total_mem: _os["default"].totalmem(),
            free_mem: _os["default"].freemem(),
            cpu: _os["default"].cpus(),
            interfaces: _os["default"].networkInterfaces()
          },
          processes: list
        };

        if (_constants["default"].WEB_STRIP_ENV_VARS === true) {
          for (var i = data.processes.length - 1; i >= 0; i--) {
            var proc = data.processes[i]; // Strip important environment variables

            if (typeof proc.pm2_env === "undefined" && typeof proc.pm2_env.env === "undefined") {
              return;
            }

            delete proc.pm2_env.env;
          }
        }

        res.statusCode = 200;
        res.write(JSON.stringify(data));
        return res.end();
      });
    } else {
      // 404
      res.statusCode = 404;
      res.write(JSON.stringify({
        err: "404"
      }));
      return res.end();
    }
  }).listen(process.env.PM2_WEB_PORT || _constants["default"].WEB_PORT, _constants["default"].WEB_IPADDR, function () {
    console.log("Web interface listening on  %s:%s", _constants["default"].WEB_IPADDR, _constants["default"].WEB_PORT);
  });
}