"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

/*
  options: {
    utimes: false,  // Boolean | Object, keep utimes if true
    mode: false,    // Boolean | Number, keep file mode if true
    cover: true,    // Boolean, cover if file exists
    filter: true,   // Boolean | Function, file filter
  }
*/
function copydirSync(from, to, options) {
  if (typeof options === "function") {
    options = {
      filter: options
    };
  }

  if (typeof options === "undefined") {
    options = {};
  }

  if (typeof options.cover === "undefined") {
    options.cover = true;
  }

  options.filter = typeof options.filter === "function" ? options.filter : function (state, filepath, filename) {
    return options.filter;
  };

  var stats = _fs["default"].lstatSync(from);

  var statsname = stats.isDirectory() ? "directory" : stats.isFile() ? "file" : stats.isSymbolicLink() ? "symbolicLink" : "";
  var valid = options.filter(statsname, from, _path["default"].dirname(from), _path["default"].basename(from));

  if (statsname === "directory" || statsname === "symbolicLink") {
    // Directory or SymbolicLink
    if (valid) {
      try {
        _fs["default"].statSync(to);
      } catch (err) {
        if (err.code === "ENOENT") {
          _fs["default"].mkdirSync(to);

          options.debug && console.log(">> " + to);
        } else {
          throw err;
        }
      }

      rewriteSync(to, options, stats);

      if (statsname != "symbolicLink") {
        listDirectorySync(from, to, options);
      }
    }
  } else if (stats.isFile()) {
    // File
    if (valid) {
      if (options.cover) {
        writeFileSync(from, to, options, stats);
      } else {
        try {
          _fs["default"].statSync(to);
        } catch (err) {
          if (err.code === "ENOENT") {
            writeFileSync(from, to, options, stats);
          } else {
            throw err;
          }
        }
      }
    }
  } else {
    throw new Error("stats invalid: " + from);
  }
}

function listDirectorySync(from, to, options) {
  var files = _fs["default"].readdirSync(from);

  copyFromArraySync(files, from, to, options);
}

function copyFromArraySync(files, from, to, options) {
  if (files.length === 0) {
    return true;
  }

  var f = files.shift();
  copydirSync(_path["default"].join(from, f), _path["default"].join(to, f), options);
  copyFromArraySync(files, from, to, options);
}

function writeFileSync(from, to, options, stats) {
  _fs["default"].writeFileSync(to, _fs["default"].readFileSync(from, "binary"), "binary");

  options.debug && console.log(">> " + to);
  rewriteSync(to, options, stats);
}

function rewriteSync(f, options, stats, callback) {
  if (options.cover) {
    var mode = options.mode === true ? stats.mode : options.mode;
    var utimes = options.utimes === true ? {
      atime: stats.atime,
      mtime: stats.mtime
    } : options.utimes;
    mode && _fs["default"].chmodSync(f, mode);
    utimes && _fs["default"].utimesSync(f, utimes.atime, utimes.mtime);
  }

  return true;
}

var _default = copydirSync;
exports["default"] = _default;